#define WANTVXDWRAPS

#include <basedef.h>
#include <vmm.h>
#include <debug.h>
#include <vxdwraps.h>
#include <vwin32.h>

#include "devicen.h"

// must be exactly 8 bytes, trailig spaces
#define DEVICE_NAME_STRING    "DEVICEx "

void *  __cdecl memcpy(void *, const void *, int);
#pragma intrinsic(memcpy)

// functions in asm module
void ControlProc( void );
void Device1ControlProc( void );
void Device2ControlProc( void );

#define NUM_DEVICES     2

extern struct VxD_Desc_Block DEVICEN_DDB; 
ULONG aControlProcs[ NUM_DEVICES ] = 
   {
      Device1ControlProc, 
      Device2ControlProc, 
   };
struct VxD_Desc_Block    aDDBs[NUM_DEVICES];
char dbgbuf[100];

BOOL VMM_Add_DDB(PVMMDDB pDDB)
{
   Touch_Register(EDI)
   _asm mov edi, pDDB
   VMMCall( VMM_Add_DDB );
   _asm jnc ok
   return FALSE;
ok:
   return TRUE;
}

BOOL VMM_Remove_DDB(PVMMDDB pDDB)
{
   Touch_Register(EDI)
   _asm mov edi, pDDB
   VMMCall( VMM_Remove_DDB );
   _asm jnc ok
   return FALSE;
ok:
   return TRUE;
}

void OnSysVmInit( DWORD hVM )
{
   int   i;

   for (i=0; i < NUM_DEVICES; i++)
   {
      memcpy( &aDDBs[i], &DEVICEN_DDB, 
              sizeof(struct VxD_Desc_Block) );
      aDDBs[i].DDB_Init_Order = UNDEFINED_INIT_ORDER;
      aDDBs[i].DDB_Control_Proc = aControlProcs[i];
      memcpy( &aDDBs[i].DDB_Name, DEVICE_NAME_STRING, 8 );
      aDDBs[i].DDB_Name[6] = i+'1';
      if (!VMM_Add_DDB( &aDDBs[i] ))
         break;
   }

}

void OnSysVmTerminate( DWORD hVM )
{
   int   i;

   for (i=0; i < NUM_DEVICES; i++)
   {
      if (!VMM_Remove_DDB( &aDDBs[i] ))
         break;
   }
}


DWORD OnW32Deviceiocontrol(ULONG DeviceNum, PDIOCPARAMETERS p)
{
   switch (p->dwIoControlCode)
   {
   case DIOC_OPEN:   
      return 0;

   case DIOC_CLOSEHANDLE:  // file closed
      return 0;

   case DIOC_DEVICEN_GETDEVICENAME:
      if (DeviceNum < NUM_DEVICES) 
      {
         if (p->cbOutBuffer < 8)
         {
            *((DWORD *)p->lpcbBytesReturned) = 0;
            return DEVICEN_RC_BAD_LENGTH;
         }
         else
         {
            memcpy( p->lpvOutBuffer, aDDBs[DeviceNum].DDB_Name, 8) ;
            *((DWORD *)p->lpcbBytesReturned) = 8;
            return 0;
         }
      }
      else 
         return DEVICEN_RC_INVALID_FUNCTION;

   case DIOC_DEVICEN_GETNUMDEVICES:
      if (DeviceNum == -1)
      {
         if (p->cbOutBuffer < sizeof(DWORD))
         {
            *((DWORD *)p->lpcbBytesReturned) = 0;
            return DEVICEN_RC_BAD_LENGTH;
         }
         else
         {
            *((DWORD *)p->lpvOutBuffer) = NUM_DEVICES;
            *((DWORD *)p->lpcbBytesReturned) = 4;
            return 0;
         }
       }
       else
         return DEVICEN_RC_INVALID_FUNCTION;

   default:
      return -1;
   }
}



